<?php

namespace FiloBlu\ExtAmastyGeoipRedirect\Controller\Change;

use FiloBlu\ExtAmastyGeoipRedirect\Helper\Data;
use Magento\Customer\Model\Session;
use Magento\Framework\App\Action\Action;
use Magento\Framework\App\Action\Context;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\App\Response\RedirectInterface;
use Magento\Framework\Controller\Result\JsonFactory;
use Magento\Store\Api\StoreCookieManagerInterface;
use Magento\Store\Model\StoreManagerInterface;
use Magento\UrlRewrite\Model\UrlFinderInterface;

class Language extends Action
{

    /** @var \Magento\Framework\View\Result\PageFactory */
    protected $resultJsonFactory;
    protected $_helper;
    protected $_storeManager;

    /**
     * @var \Magento\Customer\Model\Session
     */
    protected $customerSession;

    /**
     * @var \Magento\Store\Api\StoreCookieManagerInterface
     */
    protected $storeCookieManager;

    /** @var \Magento\Framework\App\Config\ScopeConfigInterface */
    protected $scopeConfig;

    /**
     * Language constructor.
     * @param \Magento\Framework\App\Action\Context $context
     * @param \Magento\Framework\Controller\Result\JsonFactory $resultJsonFactory
     * @param \FiloBlu\ExtAmastyGeoipRedirect\Helper\Data $helper
     * @param \Magento\Store\Model\StoreManagerInterface $storeManager
     * @param \Magento\Customer\Model\Session $customerSession
     * @param \Magento\Store\Api\StoreCookieManagerInterface $storeCookieManager
     * @param \Magento\Framework\App\Config\ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        Context $context,
        JsonFactory $resultJsonFactory,
        Data $helper,
        StoreManagerInterface $storeManager,
        Session $customerSession,
        StoreCookieManagerInterface $storeCookieManager,
        ScopeConfigInterface $scopeConfig
    ) {
        $this->resultJsonFactory = $resultJsonFactory;
        $this->_helper = $helper;
        $this->_storeManager = $storeManager;
        $this->customerSession = $customerSession;
        $this->storeCookieManager = $storeCookieManager;
        $this->scopeConfig = $scopeConfig;

        parent::__construct($context);
    }

    public function execute()
    {
        if ($this->getRequest()->isPost() && $this->getRequest()->getPost('store_id') && $this->getRequest()->getPost(
                'country_id'
            )) {
            $storeId = $this->getRequest()->getPost('store_id');
            $countryId = $this->getRequest()->getPost('country_id');
            $store = $this->_storeManager->getStore($storeId);
            $newRedirectUrl = $store->getBaseUrl();

            if (!$this->scopeConfig->getValue('amgeoipredirect/restriction/redirect_to_homepage')) {
                /** @var  \Magento\Framework\App\Response\RedirectInterface $redirect */
                $redirect = ObjectManager::getInstance()->get(RedirectInterface::class);
                /** @var  \Magento\UrlRewrite\Model\UrlFinderInterface $urlFinder */
                $urlFinder = ObjectManager::getInstance()->get(UrlFinderInterface::class);

                $refererUrl = $redirect->getRefererUrl();

                $actualUrl = null;
                foreach ($this->_storeManager->getStores() as $fromStore) {
                    $baseUrl = $fromStore->getBaseUrl();
                    if (strncasecmp($baseUrl, $refererUrl, \strlen($baseUrl)) === 0) {
                        if ($query_pos = strpos($refererUrl, '?')) {
                            $url_query_string = substr($refererUrl, $query_pos);
                            $refererUrl = str_replace($url_query_string, '', $refererUrl);
                        }
                        $actualUrl = $urlFinder->findOneByData(
                            [
                                'request_path' => str_replace($baseUrl, '', $refererUrl),
                                'store_id'     => $fromStore->getId()
                            ]
                        );

                        break;
                    }
                }

                if ($actualUrl) {
                    $destUrl = $urlFinder->findOneByData(
                        [
                            'store_id'         => $store->getId(),
                            'entity_id'        => $actualUrl->getEntityId(),
                            'entity_type'      => $actualUrl->getEntityType(),
                            'is_autogenerated' => 1
                        ]
                    );

                    $newRedirectUrl = $store->getBaseUrl() . $destUrl->getRequestPath();
                }
            }

            $this->_helper->setCountryCookie($countryId);
            $this->_helper->setStoreCookie($storeId);
            $this->storeCookieManager->setStoreCookie($store);

            $resultJson = $this->resultJsonFactory->create();

            $resultJson->setData(['url' => $newRedirectUrl]);

            return $resultJson;
        }

        return null;
    }

}
